/*  NoASM compiler for NoSense - The Virtual Machine
    Copyright (C) 2004  Adam Sawicki "Regedit"

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//-------------------------------------------------------------------------
// Name    : NoASM
// Version : 1.1
// Desc    : NoASM compiler
// Author  : Adam Sawicki "Regedit"
// Date    : 2004-06-12
// URL     : mailto:regedit@risp.pl
// URL     : http://www.programex.prv.pl/
//-------------------------------------------------------------------------
#include "stdafx.h"
#include "base.h"
#include "Tokenizer.h"
#include "Preprocessor.h"
#include "Compiler.h"

void showSyntax()
{
	std::cout << "NoASM Compiler " << VERSION_STR << " for NoSense" << std::endl;
	std::cout << "Copyright (C) 2004 by Adam Sawicki \"Regedit\"" << std::endl;
	std::cout << std::endl;
	std::cout << "Syntax:" << std::endl;
	std::cout << "  noasm.exe <SourceFile> <DestinationFile>" << std::endl;
}

// Do the compilation (on error: show error message, return false)
bool go(const std::string &fileSrc, const std::string &fileDest, bool nodisk, bool nobootinfo)
{
	g_FileSrc = fileSrc;
	g_FileDest = fileDest;
	g_PathSrc = px::ExtractFilePath(fileSrc);

	if (!LoadIncDir()) return false;
	if (!tokenize(px::GetModulePath()+"std.ninc")) return false;
	if (!tokenize(fileSrc)) return false;
	if (!preprocess()) return false;
//	debug_showSource();
	if (!compile(nodisk, nobootinfo)) return false;
	return (errorCount() == 0);
}

int main(int argc, char **argv)
{
	stats::startTime = GetTickCount();
	srand(GetTickCount());
	if (argc >= 3) {
		bool nodisk = false;
		bool nobootinfo = false;
		for (int i = 3; i < argc; i++) {
			if ( strCiEq(argv[i], "-nodisk") ) nodisk = true;
			else if ( strCiEq(argv[i], "-nobootinfo") ) nobootinfo = true;
		}
		if (go(argv[1], argv[2], nodisk, nobootinfo)) return 0;
		else return -1;
	}
	else {
		showSyntax();
		return -2;
	}
}