/*  NoASM compiler for NoSense - The Virtual Machine
    Copyright (C) 2004  Adam Sawicki "Regedit"

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//-------------------------------------------------------------------------
// Name    : Output
// Version : 1.0
// Desc    : Module for generating output file
// Author  : Adam Sawicki "Regedit"
// Date    : 2004-06-13
// URL     : mailto:regedit@risp.pl
// URL     : http://www.programex.prv.pl/
//-------------------------------------------------------------------------
#include "stdafx.h"
#include "base.h"
#include "output.h"

//-------------------------------------------------------------------------
// Private variables

// Handle to the output file
HANDLE g_file = 0;
// Name of the output file
std::string g_fileName;
// Current code address
// It is a byte counter incremented by writing functions, except header and
// boot information. It can be also modified from outside.
int g_address;
// Real code address - as above, but cannot be modified
int g_CodeSize;
// Number of reserved bytes
int g_reserved = 0;

//-------------------------------------------------------------------------
// Public functions

bool out::open(const std::string fileName)
{
	g_address = 0;
	g_CodeSize = 0;
	g_fileName = fileName;
	g_file = CreateFile(fileName.c_str(), GENERIC_WRITE, 0, 0,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
	if (g_file == INVALID_HANDLE_VALUE) {
		g_file = 0;
		showError(ERR_DESTOPEN, "Cannot open destination file");
		return false;
	}
	return true;
}

void out::close()
{
	if (g_file) {
		CloseHandle(g_file);
		g_file = 0;
	}
}

void out::cancel()
{
	if (g_file) {
		close();
		DeleteFile(g_fileName.c_str());
	}
}

bool out::opened()
{
	return (g_file != 0);
}

DWORD out::getSize()
{
	if (g_file) return GetFileSize(g_file, 0);
	else return 0;
}

int out::getAddress()
{
	return g_address;
}

void out::setAddress(int newAddress)
{
	g_address = newAddress;
}

DWORD out::getPos()
{
	if (g_file) return SetFilePointer(g_file, 0, 0, FILE_CURRENT) + g_reserved;
	else return 0;
}

void out::setPos(DWORD newPos)
{
	if (g_file) SetFilePointer(g_file, newPos, 0, FILE_BEGIN);
}

int out::getCodeSize()
{
	return g_CodeSize;
}

void out::reserve(int size)
{
	g_reserved += size;
	g_address += size;
	g_CodeSize += size;
}

bool out::writeHeader()
{
	if (g_file) {
		char header[HEADER_SIZE+1];
		strcpy(header, "NoSense Disk 10a");
		DWORD x;
		if ( !WriteFile(g_file, header, HEADER_SIZE, &x, 0) || x != HEADER_SIZE ) {
			showError(ERR_DESTWRITEHEADER, "Cannot write disk header");
			return false;
		}
	}
	return true;
}

bool out::writeBootInfo(int bootInfo)
{
	if (g_file) {
		DWORD x;
		if ( !WriteFile(g_file, &bootInfo, 4, &x, 0) || x != 4 ) {
			showError(ERR_DESTWRITEBOOTINFO, "Cannot write boot information");
			return false;
		}
	}
	return true;
}

bool out::writeCode(const void *data, int size)
{
	if (g_file) {
		DWORD x;
		// write reserved bytes
		if (g_reserved > 0) {
			BYTE ReservedBuf[BUFFER_SIZE];
			ZeroMemory(ReservedBuf, BUFFER_SIZE);
			DWORD ReservedSize;
			while (g_reserved > 0) {
				ReservedSize = static_cast<DWORD>( min(g_reserved, BUFFER_SIZE) );
				if ( !WriteFile(g_file, ReservedBuf, ReservedSize, &x, 0) || x != ReservedSize ) {
					showError(ERR_DESTWRITE, ERRMSG_DESTWRITE);
					return false;
				}
				g_reserved -= ReservedSize;
			}
		}
		// write data
		if ( !WriteFile(g_file, data, size, &x, 0) || x != size ) {
			showError(ERR_DESTWRITE, ERRMSG_DESTWRITE);
			return false;
		}
	}
	g_address += size;
	g_CodeSize += size;
	return true;
}

bool out::write(const void *data, int size)
{
	if (g_file) {
		DWORD x;
		if ( !WriteFile(g_file, data, size, &x, 0) || x != size ) {
			showError(ERR_DESTWRITE, ERRMSG_DESTWRITE);
			return false;
		}
	}
	return true;
}
