/*  NoASM compiler for NoSense - The Virtual Machine
    Copyright (C) 2004  Adam Sawicki "Regedit"

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//-------------------------------------------------------------------------
// Name    : Output
// Version : 1.0
// Desc    : Module for generating output file
// Author  : Adam Sawicki "Regedit"
// Date    : 2004-06-13
// URL     : mailto:regedit@risp.pl
// URL     : http://www.programex.prv.pl/
//-------------------------------------------------------------------------

namespace out {

const int HEADER_SIZE = 16;
const int BUFFER_SIZE = 65536;

// Open (create) output file (on error: show error, return false)
bool open(const std::string fileName);
// Close output file if it was opened
void close();
// Close output file if it was opened and delete it
void cancel();

bool opened();
DWORD getSize();
int getAddress();
void setAddress(int newAddress);
DWORD getPos();
void setPos(DWORD newPos);
int getCodeSize();

// RESERVATION virtually writes any data (eg zeros) to output file
// and moves the position, but doesn't write anything.
// Reserved bytes will be written before new code in writeCode()
// or never, if you will not call writeCode() after reservation.
void reserve(int size);

// If the file isn't open, write functions return true

// Write disk header (on error: show error, return false)
bool writeHeader();
// Write boot information (on error: show error, return false)
bool writeBootInfo(int bootInfo);
// Write data as disk content - compiled code (on error: show error, return false)
// Move position forward even if file isn't opened
bool writeCode(const void *data, int size);
// Write any data (on error: show error, return false)
// Doesn't move position.
// Use it to fill out the output file after position change.
// If file isn't opened, returns true.
bool write(const void *data, int size);

}